@@ -4,6 +4,8 @@
 #include "catalog/dependency.h"
 #include "catalog/pg_class.h"
 
+/* THIS IS USED ONLY FOR PG >= 8.5 */
+
 /*
  * Cannot include "catalog/pg_enum.h" here because we might
  * not be compiling against PG 8.5.
@@ -18,15 +20,24 @@ PG_MODULE_MAGIC;
 extern PGDLLIMPORT Oid binary_upgrade_next_pg_type_oid;
 extern PGDLLIMPORT Oid binary_upgrade_next_pg_type_array_oid;
 extern PGDLLIMPORT Oid binary_upgrade_next_pg_type_toast_oid;
+extern PGDLLIMPORT Oid binary_upgrade_next_heap_relfilenode;
+extern PGDLLIMPORT Oid binary_upgrade_next_toast_relfilenode;
+extern PGDLLIMPORT Oid binary_upgrade_next_index_relfilenode;
 
 Datum set_next_pg_type_oid(PG_FUNCTION_ARGS);
 Datum set_next_pg_type_array_oid(PG_FUNCTION_ARGS);
 Datum set_next_pg_type_toast_oid(PG_FUNCTION_ARGS);
+Datum set_next_heap_relfilenode(PG_FUNCTION_ARGS);
+Datum set_next_toast_relfilenode(PG_FUNCTION_ARGS);
+Datum set_next_index_relfilenode(PG_FUNCTION_ARGS);
 Datum add_pg_enum_label(PG_FUNCTION_ARGS);
 
 PG_FUNCTION_INFO_V1(set_next_pg_type_oid);
 PG_FUNCTION_INFO_V1(set_next_pg_type_array_oid);
 PG_FUNCTION_INFO_V1(set_next_pg_type_toast_oid);
+PG_FUNCTION_INFO_V1(set_next_heap_relfilenode);
+PG_FUNCTION_INFO_V1(set_next_toast_relfilenode);
+PG_FUNCTION_INFO_V1(set_next_index_relfilenode);
 PG_FUNCTION_INFO_V1(add_pg_enum_label);
 
 Datum
@@ -59,6 +70,36 @@ set_next_pg_type_toast_oid(PG_FUNCTION_ARGS)
 	PG_RETURN_VOID();
 }
 
+Datum
+set_next_heap_relfilenode(PG_FUNCTION_ARGS)
+{
+	Oid			relfilenode = PG_GETARG_OID(0);
+
+	binary_upgrade_next_heap_relfilenode = relfilenode;
+
+	PG_RETURN_VOID();
+}
+
+Datum
+set_next_toast_relfilenode(PG_FUNCTION_ARGS)
+{
+	Oid			relfilenode = PG_GETARG_OID(0);
+
+	binary_upgrade_next_toast_relfilenode = relfilenode;
+
+	PG_RETURN_VOID();
+}
+
+Datum
+set_next_index_relfilenode(PG_FUNCTION_ARGS)
+{
+	Oid			relfilenode = PG_GETARG_OID(0);
+
+	binary_upgrade_next_index_relfilenode = relfilenode;
+
+	PG_RETURN_VOID();
+}
+
 Datum
 add_pg_enum_label(PG_FUNCTION_ARGS)
 {
