@@ -121,7 +121,6 @@ checkBinDir(migratorContext *ctx, ClusterInfo *cluster)
 	if (check_exec(ctx, cluster->bindir, "psql", NULL) == 1)
 #endif
 		cluster->psql_exe = "psql";
-	check_ok(ctx);
 }
 
 
@@ -136,25 +135,17 @@ checkBinDir(migratorContext *ctx, ClusterInfo *cluster)
 void
 verify_directories(migratorContext *ctx)
 {
-	prep_status(ctx, "Checking old data directory %s", ctx->old.pgdata);
-	if (check_data_dir(ctx, ctx->old.pgdata) == 0)
-		check_ok(ctx);
-	else
+	prep_status(ctx, "Checking old data directory (%s)", ctx->old.pgdata);
+	if (check_data_dir(ctx, ctx->old.pgdata) != 0)
 		pg_log(ctx, PG_FATAL, "Failed\n");
-
-	prep_status(ctx, "Checking new data directory %s", ctx->new.pgdata);
-	if (check_data_dir(ctx, ctx->new.pgdata) == 0)
-		check_ok(ctx);
-	else
-		pg_log(ctx, PG_FATAL, "Failed\n");
-
-	/* verify that postmasters exist */
-
-	prep_status(ctx, "Checking binaries in old cluster (%s)", ctx->old.bindir);
 	checkBinDir(ctx, &ctx->old);
+	check_ok(ctx);
 
-	prep_status(ctx, "Checking binaries in new cluster (%s)", ctx->new.bindir);
+	prep_status(ctx, "Checking new data directory (%s)", ctx->new.pgdata);
+	if (check_data_dir(ctx, ctx->new.pgdata) != 0)
+		pg_log(ctx, PG_FATAL, "Failed\n");
 	checkBinDir(ctx, &ctx->new);
+	check_ok(ctx);
 }
 
 
