@@ -89,8 +89,8 @@ transfer_single_new_db(migratorContext *ctx, pageCnvCtx *pageConverter,
 					maps[mapnum].old_nspname, maps[mapnum].old_relname,
 					maps[mapnum].new_nspname, maps[mapnum].new_relname);
 
-		if (GET_MAJOR_VERSION(ctx->old.pg_version) >= 804 &&
-			GET_MAJOR_VERSION(ctx->new.pg_version) >= 804)
+		if (GET_MAJOR_VERSION(ctx->old.major_version) >= 804 &&
+			GET_MAJOR_VERSION(ctx->new.major_version) >= 804)
 		{
 			/*
 			 * Now copy/link any fsm and vm files, if they exist
@@ -320,8 +320,8 @@ copy_dir(const char *src, const char *dst, bool force)
 
 	while ((de = readdir(srcdir)) != NULL)
 	{
-		char		src_file[MAX_STRING];
-		char		dest_file[MAX_STRING];
+		char		src_file[MAXPGPATH];
+		char		dest_file[MAXPGPATH];
 
 		if (strcmp(de->d_name, ".") == 0 || strcmp(de->d_name, "..") == 0)
 			continue;
