@@ -188,7 +188,7 @@ map_rel_by_id(migratorContext *ctx, Oid oldid, Oid newid,
 	if (strlen(old_tablespace) == 0)
 	{
 		/*
-		 * relation does not belong to the default tablespace, hence relfiles
+		 * relation belongs to the default tablespace, hence relfiles
 		 * would exist in the data directories.
 		 */
 		snprintf(map->old_file, sizeof(map->old_file), "%s/base/%u", olddata, old_db->db_oid);
@@ -200,8 +200,10 @@ map_rel_by_id(migratorContext *ctx, Oid oldid, Oid newid,
 		 * relation belongs to some tablespace, hence copy its physical
 		 * location
 		 */
-		snprintf(map->old_file, sizeof(map->old_file), "%s.old/%u", old_tablespace, old_db->db_oid);
-		snprintf(map->new_file, sizeof(map->new_file), "%s/%u", old_tablespace, new_db->db_oid);
+		snprintf(map->old_file, sizeof(map->old_file), "%s%s/%u", old_tablespace,
+				ctx->old.tablespace_suffix, old_db->db_oid);
+		snprintf(map->new_file, sizeof(map->new_file), "%s%s/%u", old_tablespace,
+				 ctx->new.tablespace_suffix, new_db->db_oid);
 	}
 }
 
@@ -371,8 +373,8 @@ get_rel_infos(migratorContext *ctx, const char *db_name, RelInfoArr *relarr,
 				"ORDER BY n.nspname, c.relname;",
 				FirstNormalObjectId,
 				/* see the comment at the top of v8_3_create_sequence_script() */
-				(GET_MAJOR_VERSION(ctx->old.pg_version) <= 803 &&
-				 GET_MAJOR_VERSION(ctx->new.pg_version) >= 804) ?
+				(GET_MAJOR_VERSION(ctx->old.major_version) <= 803 &&
+				 GET_MAJOR_VERSION(ctx->new.major_version) >= 804) ?
 					"" : " OR relkind = 'S'",
 				/*
 				 *	EDB AS installs pgagent by default via initdb.
@@ -553,7 +555,7 @@ check_loadable_libraries(migratorContext *ctx)
 	prep_status(ctx, "Checking for presence of required libraries");
 
 	snprintf(output_path, sizeof(output_path), "%s/loadable_libraries.txt",
-			ctx->home_dir);
+			ctx->output_dir);
 	
 	for (libnum = 0; libnum < ctx->num_libraries; libnum++)
 	{
