@@ -6,6 +6,8 @@
 
 #include "pg_migrator.h"
 
+#include <sys/stat.h>
+
 #ifdef HAVE_LANGINFO_H
 #include <langinfo.h>
 #endif
@@ -28,9 +30,10 @@ static void check_cluster_compatibility(migratorContext *ctx, bool live_check);
 static void setup(migratorContext *ctx, char *argv0, bool live_check);
 static void cleanup(migratorContext *ctx);
 static void rename_old_pg_control(migratorContext *ctx);
+static void create_empty_output_directory(migratorContext *ctx);
 
 /* OID system catalog preservation added during PG 8.5 development */
-#define OID_PRESERVATION_CAT_VER	200912271
+#define TABLE_SPACE_SUBDIRS	201001111
 
 /*
  *	main
@@ -40,6 +43,7 @@ main(int argc, char **argv)
 {
 	migratorContext ctx;
 	char *sequence_script_file_name = NULL;
+	char *deletion_script_file_name = NULL;
 	bool	live_check = false;
 	
 	memset(&ctx, 0, sizeof(ctx));
@@ -63,6 +67,8 @@ main(int argc, char **argv)
 
 	setup(&ctx, argv[0], live_check);
 
+	create_empty_output_directory(&ctx);
+
 	check_cluster_versions(&ctx);
 	check_cluster_compatibility(&ctx, live_check);
 
@@ -86,7 +92,7 @@ main(int argc, char **argv)
 	 *	Check for various failure cases
 	 */
 
-	if (ctx.new.controldata.cat_ver < OID_PRESERVATION_CAT_VER)
+	if (GET_MAJOR_VERSION(ctx.new.major_version) <= 804)
 	{
 		v_8_4_check_for_composite_types(&ctx, CLUSTER_OLD);
 		v_8_4_check_for_array_types(&ctx, CLUSTER_OLD);
@@ -95,8 +101,8 @@ main(int argc, char **argv)
 
 	v8_3_check_for_isn_and_int8_passing_mismatch(&ctx, CLUSTER_OLD);
 	
-	if (GET_MAJOR_VERSION(ctx.old.pg_version) <= 803 &&
-		GET_MAJOR_VERSION(ctx.new.pg_version) >= 804)
+	if (GET_MAJOR_VERSION(ctx.old.major_version) <= 803 &&
+		GET_MAJOR_VERSION(ctx.new.major_version) >= 804)
 	{
 		v8_3_check_for_name_data_type_usage(&ctx, CLUSTER_OLD);
 		v8_3_check_for_tsquery_usage(&ctx, CLUSTER_OLD);
@@ -115,11 +121,14 @@ main(int argc, char **argv)
 			sequence_script_file_name =
 				v8_3_create_sequence_script(&ctx, CLUSTER_OLD);
 	}
-	if (GET_MAJOR_VERSION(ctx.old.pg_version) <= 804 &&
-		GET_MAJOR_VERSION(ctx.new.pg_version) >= 805)
+	if (GET_MAJOR_VERSION(ctx.old.major_version) <= 804 &&
+		GET_MAJOR_VERSION(ctx.new.major_version) >= 805)
 		v8_4_populate_pg_largeobject_metadata(&ctx, true, CLUSTER_OLD);
 
-	/* While not a check option, we do this while the old server is running */
+	/*
+	 * While not a check option, we do this now because this is the only
+	 * time the old server is running.
+	 */
 	if (!ctx.check)
 	{
 		generate_old_dump(&ctx);
@@ -156,8 +165,9 @@ main(int argc, char **argv)
 		"| If pg_migrator fails after this point, you must\n"
 		"| re-initdb the new cluster before continuing.\n"
 		"| You will also need to remove the \".old\" suffix\n"
-		"| from old $PGDATA/global/pg_control");
-	if (ctx.num_tablespaces)
+		"| from %s/global/pg_control.old", ctx.old.pgdata);
+
+	if (GET_MAJOR_VERSION(ctx.new.major_version) <= 804 && ctx.num_tablespaces)
 		pg_log(&ctx, PG_REPORT,
 			" and old\n| tablespace directory names");
 	pg_log(&ctx, PG_REPORT, ".\n");
@@ -165,6 +175,17 @@ main(int argc, char **argv)
 	pg_log(&ctx, PG_REPORT, "\nPerforming Migration\n");
 	pg_log(&ctx, PG_REPORT, "--------------------\n");
 	
+	/*
+	 *	Rename all tablespace paths to .old so we can copy them later.
+	 *	This is more likely to fail than rename_old_pg_control() so do
+	 *	it first.
+	 */
+	if (GET_MAJOR_VERSION(ctx.new.major_version) <= 804 && ctx.num_tablespaces)
+		rename_tablespaces(&ctx);
+
+	set_tablespace_subdirectory(&ctx, CLUSTER_OLD);
+	set_tablespace_subdirectory(&ctx, CLUSTER_NEW);
+
 	/*
 	 *	Destructive Changes to Old Cluster (New cluster still running)
 	 */
@@ -172,12 +193,6 @@ main(int argc, char **argv)
 	/* rename pg_control so old server cannot be accidentally started */
 	rename_old_pg_control(&ctx);
 	
-	/*
-	 *	Rename all tablespace paths to .old so we can copy them later.
-	 */
-	rename_tablespaces(&ctx);
-	check_ok(&ctx);
-
 	/*
 	 *	Non-Destructive Changes to New Cluster
 	 */
@@ -233,18 +248,20 @@ main(int argc, char **argv)
 			  SYSTEMQUOTE "\"%s/%s\" --set ON_ERROR_STOP=on --port %d "
 			  "-f \"%s/%s\" --dbname template1 >> \"%s\"" SYSTEMQUOTE,
 			  ctx.new.bindir, ctx.new.psql_exe, ctx.new.port,
-			  ctx.home_dir, GLOBALS_DUMP_FILE, ctx.logfile);
+			  ctx.output_dir, GLOBALS_DUMP_FILE, ctx.logfile);
 	check_ok(&ctx);
 
 	get_db_and_rel_infos(&ctx, &ctx.new.dbarr, CLUSTER_NEW);
 
 	stop_postmaster(&ctx, false, false);
 
-	prep_status(&ctx, "Creating placeholder relfiles for toast relations\n");
-	reserve_relfilenode_names(&ctx);
-	prep_status(&ctx, "");
-	check_ok(&ctx);
-
+	if (GET_MAJOR_VERSION(ctx.new.major_version) <= 804)
+	{
+		prep_status(&ctx, "Creating placeholder relfiles for toast relations\n");
+			reserve_relfilenodes(&ctx);
+		prep_status(&ctx, "");
+		check_ok(&ctx);
+	}
 
 	/* -- NEW -- */
 	start_postmaster(&ctx, CLUSTER_NEW, false);
@@ -256,27 +273,30 @@ main(int argc, char **argv)
 			  SYSTEMQUOTE "\"%s/%s\" --set ON_ERROR_STOP=on --port %d "
 			  "-f \"%s/%s\" --dbname template1 >> \"%s\"" SYSTEMQUOTE,
 			  ctx.new.bindir, ctx.new.psql_exe, ctx.new.port,
-			  ctx.home_dir, DB_DUMP_FILE, ctx.logfile);
+			  ctx.output_dir, DB_DUMP_FILE, ctx.logfile);
 	check_ok(&ctx);
 
 	/* regenerate now that we have db schemas */
 	dbarr_free(&ctx.new.dbarr);
 	get_db_and_rel_infos(&ctx, &ctx.new.dbarr, CLUSTER_NEW);
 
-	prep_status(&ctx, "Restoring relations to use fixed toast file names\n");
-	adjust_rels_to_use_fixed_toast_relfilenodes(&ctx, &ctx.old.dbarr, &ctx.new.dbarr);
-	prep_status(&ctx, "");	/* in case nothing printed */
-	check_ok(&ctx);
+	if (GET_MAJOR_VERSION(ctx.new.major_version) <= 804)
+	{
+		prep_status(&ctx, "Restoring relations to use fixed toast file names\n");
+		adjust_rels_to_use_fixed_toast_relfilenodes(&ctx, &ctx.old.dbarr, &ctx.new.dbarr);
+		prep_status(&ctx, "");	/* in case nothing printed */
+		check_ok(&ctx);
+
+		/*
+		 * adjust_rels_to_use_fixed_toast_relfilenodes would have created new
+		 * entries in the new database; hence regenerate new_dbinfos to get in sync.
+		 */
+		dbarr_free(&ctx.new.dbarr);
+		get_db_and_rel_infos(&ctx, &ctx.new.dbarr, CLUSTER_NEW);
+	}
 
 	uninstall_support_functions(&ctx);
 
-	/*
-	 * adjust_rels_to_use_fixed_toast_relfilenodes would have created new
-	 * entries in the new database; hence regenerate new_dbinfos to get in sync.
-	 */
-	dbarr_free(&ctx.new.dbarr);
-	get_db_and_rel_infos(&ctx, &ctx.new.dbarr, CLUSTER_NEW);
-
 	stop_postmaster(&ctx, false, false);
 
 
@@ -297,8 +317,10 @@ main(int argc, char **argv)
 		  ctx.new.bindir, ctx.old.controldata.chkpnt_nxtoid, ctx.new.pgdata);
 	check_ok(&ctx);
 
-	if (GET_MAJOR_VERSION(ctx.old.pg_version) <= 803 &&
-		GET_MAJOR_VERSION(ctx.new.pg_version) >= 804)
+	deletion_script_file_name = create_script_for_old_cluster_deletion(&ctx);
+	
+	if (GET_MAJOR_VERSION(ctx.old.major_version) <= 803 &&
+		GET_MAJOR_VERSION(ctx.new.major_version) >= 804)
 	{
 		start_postmaster(&ctx, CLUSTER_NEW, true);
 
@@ -322,29 +344,38 @@ main(int argc, char **argv)
 		stop_postmaster(&ctx, false, true);
 	}
 
-	if (GET_MAJOR_VERSION(ctx.old.pg_version) <= 804 &&
-		GET_MAJOR_VERSION(ctx.new.pg_version) >= 805)
+	if (GET_MAJOR_VERSION(ctx.old.major_version) <= 804 &&
+		GET_MAJOR_VERSION(ctx.new.major_version) >= 805)
 	{
 		start_postmaster(&ctx, CLUSTER_NEW, true);
 		v8_4_populate_pg_largeobject_metadata(&ctx, false, CLUSTER_NEW);
 		stop_postmaster(&ctx, false, true);
 	}
 	
-	pg_log(&ctx, PG_REPORT, "*Upgrade complete*\n");
+	
+	pg_log(&ctx, PG_REPORT, "\nUpgrade complete\n");
+	pg_log(&ctx, PG_REPORT, "----------------\n");
 
-	if (GET_MAJOR_VERSION(ctx.old.pg_version) >= 804 &&
-		GET_MAJOR_VERSION(ctx.new.pg_version) >= 804)
-		pg_log(&ctx, PG_REPORT, "\n"
-			   "| Optimizer statistics is not transferred by pg_migrator\n"
-			   "| so consider running ANALYZE on all databases in the\n"
-			   "| newly-upgraded cluster.\n");
+	if (GET_MAJOR_VERSION(ctx.old.major_version) >= 804 &&
+		GET_MAJOR_VERSION(ctx.new.major_version) >= 804)
+		pg_log(&ctx, PG_REPORT,
+			    "| Optimizer statistics is not transferred by pg_migrator\n"
+			    "| so consider running:\n"
+				"| \tvacuumdb --all --analyze-only\n" 
+				"| on the newly-upgraded cluster.\n\n");
 	else
-		pg_log(&ctx, PG_REPORT, "\n"
+		pg_log(&ctx, PG_REPORT,
 		  		"| Optimizer statistics and free space information\n"
 				"| are not transferred by pg_migrator so consider\n"
 				"| running:\n"
-				"| \tvacuumdb --all --analyze\n" 
-				"| on the newly-upgraded cluster.\n");
+				"| \tvacuumdb --all --analyze\n"
+				"| on the newly-upgraded cluster.\n\n");
+
+	pg_log(&ctx, PG_REPORT,
+				"| Running this script will delete the old cluster's data files:\n"
+				"| \t%s\n",
+				deletion_script_file_name);
+	pg_free(deletion_script_file_name);
 
 	cleanup(&ctx);
 
@@ -368,7 +399,7 @@ generate_old_dump(migratorContext *ctx)
 	exec_prog(ctx, true,
 			SYSTEMQUOTE "\"%s/pg_dumpall\" --port %d --schema-only "
 			"--binary-upgrade > \"%s/" ALL_DUMP_FILE "\"" SYSTEMQUOTE,
-		    ctx->new.bindir, ctx->old.port, ctx->home_dir);
+		    ctx->new.bindir, ctx->old.port, ctx->output_dir);
 	check_ok(ctx);
 }
 
@@ -395,16 +426,16 @@ split_old_dump(migratorContext *ctx)
 	bool	start_of_line = true;
 	char	create_role_str[MAX_STRING];
 	char	create_role_str_quote[MAX_STRING];
-	char	filename[MAX_STRING];
+	char	filename[MAXPGPATH];
 	bool	suppressed_username = false;
 	
-	snprintf(filename, sizeof(filename), "%s/%s", ctx->home_dir, ALL_DUMP_FILE);
+	snprintf(filename, sizeof(filename), "%s/%s", ctx->output_dir, ALL_DUMP_FILE);
 	if ((all_dump =	fopen(filename, "r")) == NULL)
 		pg_log(ctx, PG_FATAL, "Cannot open dump file %s\n", filename);
-	snprintf(filename, sizeof(filename), "%s/%s", ctx->home_dir, GLOBALS_DUMP_FILE);
+	snprintf(filename, sizeof(filename), "%s/%s", ctx->output_dir, GLOBALS_DUMP_FILE);
 	if ((globals_dump =	fopen(filename, "w")) == NULL)
 		pg_log(ctx, PG_FATAL, "Cannot write to dump file %s\n", filename);
-	snprintf(filename, sizeof(filename), "%s/%s", ctx->home_dir, DB_DUMP_FILE);
+	snprintf(filename, sizeof(filename), "%s/%s", ctx->output_dir, DB_DUMP_FILE);
 	if ((db_dump =	fopen(filename, "w")) == NULL)
 		pg_log(ctx, PG_FATAL, "Cannot write to dump file %s\n", filename);
 	current_output = globals_dump;
@@ -601,23 +632,23 @@ static void
 check_cluster_versions(migratorContext *ctx)
 {
 	/* get old and new cluster versions */
-	ctx->old.pg_version = get_postgres_version(ctx, &ctx->old.pg_version_str, CLUSTER_OLD);
-	ctx->new.pg_version = get_postgres_version(ctx, &ctx->new.pg_version_str, CLUSTER_NEW);
+	ctx->old.major_version = get_major_server_version(ctx, &ctx->old.major_version_str, CLUSTER_OLD);
+	ctx->new.major_version = get_major_server_version(ctx, &ctx->new.major_version_str, CLUSTER_NEW);
 
 	/* We allow migration from/to the same major version for beta upgrades */
 
-	if (GET_MAJOR_VERSION(ctx->old.pg_version) < 803)
+	if (GET_MAJOR_VERSION(ctx->old.major_version) < 803)
 		pg_log(ctx, PG_FATAL, "This utility can only upgrade from PostgreSQL version 8.3 and later.\n");
 
 	if (!ctx->new.is_edb_as)
 	{
 		/* Only PG 8.4+ has enhancements to pg_dump, pg_dumpall, and vacuumdb */
-		if (GET_MAJOR_VERSION(ctx->new.pg_version) < 804)
+		if (GET_MAJOR_VERSION(ctx->new.major_version) < 804)
 			pg_log(ctx, PG_FATAL, "This utility can only upgrade to PostgreSQL version 8.4 and later.\n");
 	}
 	else
 	{
-		if (GET_MAJOR_VERSION(ctx->new.pg_version) < 803)
+		if (GET_MAJOR_VERSION(ctx->new.major_version) < 803)
 			pg_log(ctx, PG_FATAL, "This utility can only upgrade to EnterpriseDB Postgres Plus Advanced Server\nversion 8.3 and later.\n");
 	}
 
@@ -625,21 +656,21 @@ check_cluster_versions(migratorContext *ctx)
 	 * We can't allow downgrading because we use the target pg_dumpall, and
 	 * pg_dumpall cannot operate on new datbase versions, only older versions.
 	 */
-	if (ctx->old.pg_version > ctx->new.pg_version)
+	if (ctx->old.major_version > ctx->new.major_version)
 		pg_log(ctx, PG_FATAL, "This utility cannot be used to downgrade to older major PostgreSQL versions.\n");
 
-	/* Must be built using _new_ server source */
+	/* Must be built using _new_ server source because PG_MODULE_MAGIC must match new server. */
 	if (GET_MAJOR_VERSION(PG_VERSION_NUM) !=
-		GET_MAJOR_VERSION(ctx->new.pg_version))
+		GET_MAJOR_VERSION(ctx->new.major_version))
 		pg_log(ctx, PG_FATAL,
 				"This binary was built using %s PostgreSQL sources; it must be\n"
 				"built using %s PostgreSQL sources to match the new cluster.\n",
 #if defined(PG_MAJORVERSION) || !defined(EDB_NATIVE_LANG)
 				PG_MAJORVERSION,
 #else
-				"8.3",	/* assume EDB AS 8.3 */
+				"8.3",	/* PG_MAJORVERSION added in 8.4; assume EDB AS 8.3 */
 #endif
-				ctx->new.pg_version_str);
+				ctx->new.major_version_str);
 }
 
 
@@ -653,10 +684,10 @@ check_cluster_compatibility(migratorContext *ctx, bool live_check)
 	FILE 		*lib_test;
 
 	/*
-	 *	Test pg_migrator.so is in the proper place.   We cannot copy it
+	 *	Test pg_migrator_sysoids.so is in the proper place.   We cannot copy it
 	 *	ourselves because install directories are typically root-owned.
 	 */
-	snprintf(libfile, sizeof(libfile), "%s/pg_migrator%s", ctx->new.libpath,
+	snprintf(libfile, sizeof(libfile), "%s/pg_migrator_sysoids%s", ctx->new.libpath,
 				DLSUFFIX);
 
 	if ((lib_test = fopen(libfile, "r")) == NULL)
@@ -669,6 +700,10 @@ check_cluster_compatibility(migratorContext *ctx, bool live_check)
 	get_control_data(ctx, &ctx->old, live_check);
 	get_control_data(ctx, &ctx->new, false);
 	check_control_data(ctx, &ctx->old.controldata, &ctx->new.controldata);
+	if (GET_MAJOR_VERSION(ctx->new.major_version) == 805 &&
+		ctx->new.controldata.cat_ver < TABLE_SPACE_SUBDIRS)
+		pg_log(ctx, PG_FATAL, "This utility can only upgrade to PostgreSQL version 8.5 after 2010-01-11\n"
+							  "because of backend API changes made during development.\n");
 }
 
 
@@ -679,7 +714,7 @@ static void
 cleanup(migratorContext *ctx)
 {
 	int			tblnum;
-	char	filename[MAX_STRING];
+	char	filename[MAXPGPATH];
 
 	for (tblnum = 0; tblnum < ctx->num_tablespaces; tblnum++)
 		pg_free(ctx->tablespaces[tblnum]);
@@ -689,14 +724,16 @@ cleanup(migratorContext *ctx)
 	dbarr_free(&ctx->new.dbarr);
 	pg_free(ctx->logfile);
 	pg_free(ctx->user);
-	pg_free(ctx->old.pg_version_str);
-	pg_free(ctx->new.pg_version_str);
+	pg_free(ctx->old.major_version_str);
+	pg_free(ctx->new.major_version_str);
 	pg_free(ctx->old.controldata.lc_collate);
 	pg_free(ctx->new.controldata.lc_collate);
 	pg_free(ctx->old.controldata.lc_ctype);
 	pg_free(ctx->new.controldata.lc_ctype);
 	pg_free(ctx->old.controldata.encoding);
 	pg_free(ctx->new.controldata.encoding);
+	pg_free(ctx->old.tablespace_suffix);
+	pg_free(ctx->new.tablespace_suffix);
 	
 	if (ctx->log_fd != NULL)
 	{
@@ -707,11 +744,11 @@ cleanup(migratorContext *ctx)
 	if (ctx->debug_fd)
 		fclose(ctx->debug_fd);
 
-	snprintf(filename, sizeof(filename), "%s/%s", ctx->home_dir, ALL_DUMP_FILE);
+	snprintf(filename, sizeof(filename), "%s/%s", ctx->output_dir, ALL_DUMP_FILE);
 	unlink(filename);
-	snprintf(filename, sizeof(filename), "%s/%s", ctx->home_dir, GLOBALS_DUMP_FILE);
+	snprintf(filename, sizeof(filename), "%s/%s", ctx->output_dir, GLOBALS_DUMP_FILE);
 	unlink(filename);
-	snprintf(filename, sizeof(filename), "%s/%s", ctx->home_dir, DB_DUMP_FILE);
+	snprintf(filename, sizeof(filename), "%s/%s", ctx->output_dir, DB_DUMP_FILE);
 	unlink(filename);
 }
 
@@ -745,17 +782,20 @@ install_support_functions(migratorContext *ctx)
 		PQclear(executeQueryOrDie(ctx, conn,
 					"CREATE SCHEMA binary_upgrade;"));
 
-		PQclear(executeQueryOrDie(ctx, conn,
-					"CREATE OR REPLACE FUNCTION binary_upgrade.pg_toasttbl_recreate(OID,OID) "
-					"RETURNS VOID "
-					"AS '$libdir/pg_migrator' "
-					"LANGUAGE C STRICT;"));
-		PQclear(executeQueryOrDie(ctx, conn,
-					"CREATE OR REPLACE FUNCTION binary_upgrade.pg_toasttbl_drop(OID) "
-					"RETURNS VOID "
-					"AS '$libdir/pg_migrator' "
-					"LANGUAGE C STRICT;"));
-		if (ctx->new.controldata.cat_ver >= OID_PRESERVATION_CAT_VER)
+		if (GET_MAJOR_VERSION(ctx->new.major_version) <= 804)
+		{
+			PQclear(executeQueryOrDie(ctx, conn,
+						"CREATE OR REPLACE FUNCTION binary_upgrade.pg_toasttbl_recreate(OID,OID) "
+						"RETURNS VOID "
+						"AS '$libdir/pg_migrator_toast' "
+						"LANGUAGE C STRICT;"));
+			PQclear(executeQueryOrDie(ctx, conn,
+						"CREATE OR REPLACE FUNCTION binary_upgrade.pg_toasttbl_drop(OID) "
+						"RETURNS VOID "
+						"AS '$libdir/pg_migrator_toast' "
+						"LANGUAGE C STRICT;"));
+		}
+		else
 		{
 			PQclear(executeQueryOrDie(ctx, conn,
 						"CREATE OR REPLACE FUNCTION "
@@ -775,6 +815,24 @@ install_support_functions(migratorContext *ctx)
 						"RETURNS VOID "
 						"AS '$libdir/pg_migrator_sysoids' "
 						"LANGUAGE C STRICT;"));
+			PQclear(executeQueryOrDie(ctx, conn,
+						"CREATE OR REPLACE FUNCTION "
+						"		binary_upgrade.set_next_heap_relfilenode(OID) "
+						"RETURNS VOID "
+						"AS '$libdir/pg_migrator_sysoids' "
+						"LANGUAGE C STRICT;"));
+			PQclear(executeQueryOrDie(ctx, conn,
+						"CREATE OR REPLACE FUNCTION "
+						"		binary_upgrade.set_next_toast_relfilenode(OID) "
+						"RETURNS VOID "
+						"AS '$libdir/pg_migrator_sysoids' "
+						"LANGUAGE C STRICT;"));
+			PQclear(executeQueryOrDie(ctx, conn,
+						"CREATE OR REPLACE FUNCTION "
+						"		binary_upgrade.set_next_index_relfilenode(OID) "
+						"RETURNS VOID "
+						"AS '$libdir/pg_migrator_sysoids' "
+						"LANGUAGE C STRICT;"));
 			PQclear(executeQueryOrDie(ctx, conn,
 						"CREATE OR REPLACE FUNCTION "
 						"		binary_upgrade.add_pg_enum_label(OID, OID, NAME) "
@@ -823,7 +881,7 @@ uninstall_support_functions(migratorContext *ctx)
 static void
 rename_old_pg_control(migratorContext *ctx)
 {
-	char	old_path[MAX_STRING], new_path[MAX_STRING];
+	char	old_path[MAXPGPATH], new_path[MAXPGPATH];
 
 	prep_status(ctx, "Adding \".old\" suffix to old global/pg_control");
 
@@ -838,3 +896,20 @@ rename_old_pg_control(migratorContext *ctx)
 	}
 	check_ok(ctx);
 }
+
+
+static void
+create_empty_output_directory(migratorContext *ctx)
+{
+	/* this might fail if the directory does not exist */
+	rmtree(ctx->output_dir, true);
+	
+	if (mkdir(ctx->output_dir, S_IRWXU) != 0)
+	{
+#ifdef WIN32
+		_dosmaperr(GetLastError());
+#endif
+		pg_log(ctx, PG_FATAL, "Cannot create subdirectory %s: %s\n",
+			ctx->output_dir, strerror(errno));
+	}
+}
