@@ -28,7 +28,7 @@ v8_3_check_for_name_data_type_usage(migratorContext *ctx, Cluster whichCluster)
 	prep_status(ctx, "Checking for invalid 'name' user columns");
 
 	snprintf(output_path, sizeof(output_path), "%s/tables_using_name.txt",
-			ctx->home_dir);
+			ctx->output_dir);
 	
 	for (dbnum = 0; dbnum < active_cluster->dbarr.ndbs; dbnum++)
 	{
@@ -120,7 +120,7 @@ v8_3_check_for_tsquery_usage(migratorContext *ctx, Cluster whichCluster)
 	prep_status(ctx, "Checking for tsquery user columns");
 
 	snprintf(output_path, sizeof(output_path), "%s/tables_using_tsquery.txt",
-			ctx->home_dir);
+			ctx->output_dir);
 	
 	for (dbnum = 0; dbnum < active_cluster->dbarr.ndbs; dbnum++)
 	{
@@ -218,7 +218,7 @@ v8_3_check_for_isn_and_int8_passing_mismatch(migratorContext *ctx, Cluster which
 	}
 
 	snprintf(output_path, sizeof(output_path), "%s/contrib_isn_and_int8_pass_by_value.txt",
-			ctx->home_dir);
+			ctx->output_dir);
 	
 	for (dbnum = 0; dbnum < active_cluster->dbarr.ndbs; dbnum++)
 	{
@@ -307,7 +307,7 @@ v8_3_rebuild_tsvector_tables(migratorContext *ctx, bool check_mode,
 	prep_status(ctx, "Checking for tsvector user columns");
 
 	snprintf(output_path, sizeof(output_path), "%s/rebuild_tsvector_tables.sql",
-			ctx->home_dir);
+			ctx->output_dir);
 
 	for (dbnum = 0; dbnum < active_cluster->dbarr.ndbs; dbnum++)
 	{
@@ -449,7 +449,7 @@ v8_3_invalidate_hash_gin_indexes(migratorContext *ctx, bool check_mode,
 	prep_status(ctx, "Checking for hash and gin indexes");
 
 	snprintf(output_path, sizeof(output_path), "%s/reindex_hash_and_gin.sql",
-			ctx->home_dir);
+			ctx->output_dir);
 
 	for (dbnum = 0; dbnum < active_cluster->dbarr.ndbs; dbnum++)
 	{
@@ -564,7 +564,7 @@ v8_3_invalidate_bpchar_pattern_ops_indexes(migratorContext *ctx, bool check_mode
 	prep_status(ctx, "Checking for bpchar_pattern_ops indexes");
 
 	snprintf(output_path, sizeof(output_path), "%s/reindex_bpchar_ops.sql",
-			ctx->home_dir);
+			ctx->output_dir);
 
 	for (dbnum = 0; dbnum < active_cluster->dbarr.ndbs; dbnum++)
 	{
@@ -693,7 +693,7 @@ v8_4_populate_pg_largeobject_metadata(migratorContext *ctx, bool check_mode,
 	prep_status(ctx, "Checking for large objects");
 
 	snprintf(output_path, sizeof(output_path), "%s/pg_largeobject.sql",
-			ctx->home_dir);
+			ctx->output_dir);
 
 	for (dbnum = 0; dbnum < active_cluster->dbarr.ndbs; dbnum++)
 	{
@@ -775,7 +775,7 @@ v8_3_create_sequence_script(migratorContext *ctx, Cluster whichCluster)
 	bool		found = false;
 	char		*output_path = pg_malloc(ctx, MAXPGPATH);
 
-	snprintf(output_path, MAXPGPATH, "%s/adjust_sequences.sql", ctx->home_dir);
+	snprintf(output_path, MAXPGPATH, "%s/adjust_sequences.sql", ctx->output_dir);
 
 	prep_status(ctx, "Creating script to adjust sequences");
 
@@ -879,7 +879,7 @@ v_8_4_check_for_composite_types(migratorContext *ctx, Cluster whichCluster)
 	prep_status(ctx, "Checking for columns with user-defined composite types");
 
 	snprintf(output_path, sizeof(output_path), "%s/tables_using_composite_types.txt",
-			ctx->home_dir);
+			ctx->output_dir);
 	
 	for (dbnum = 0; dbnum < active_cluster->dbarr.ndbs; dbnum++)
 	{
@@ -973,7 +973,7 @@ v_8_4_check_for_array_types(migratorContext *ctx, Cluster whichCluster)
 	prep_status(ctx, "Checking for columns with user-defined array types");
 
 	snprintf(output_path, sizeof(output_path), "%s/tables_using_array_types.txt",
-			ctx->home_dir);
+			ctx->output_dir);
 	
 	for (dbnum = 0; dbnum < active_cluster->dbarr.ndbs; dbnum++)
 	{
@@ -1071,7 +1071,7 @@ v_8_4_check_for_enum_types(migratorContext *ctx, Cluster whichCluster)
 	prep_status(ctx, "Checking for columns with user-defined enum types");
 
 	snprintf(output_path, sizeof(output_path), "%s/tables_using_enum_types.txt",
-			ctx->home_dir);
+			ctx->output_dir);
 	
 	for (dbnum = 0; dbnum < active_cluster->dbarr.ndbs; dbnum++)
 	{
